@@ -18,6 +18,7 @@ from pathlib import Path
 import sys
 from argparse import ArgumentParser
 from argparse import RawDescriptionHelpFormatter
+import json
 
 class WikiPush(object):
     '''
@@ -53,13 +54,51 @@ class WikiPush(object):
             
     def formatQueryResult(self,askQuery,wiki=None,limit=None,showProgress=False,queryDivision=1,outputFormat='json'):
         '''
-        format the query result for the given askQuery
+        format the query result for the given askQuery.
+        Args:
+             askQuery(string): Semantic Media Wiki in line query https://www.semantic-mediawiki.org/wiki/Help:Inline_queries
+            wiki(wikibot): the wiki to query - use fromWiki if not specified
+            limit(int): the limit for the query (optional)
+            showProgress(bool): true if progress of the query retrieval should be indicated (default: one dot per 50 records ...)
+            queryDivision(int): Defines the number of subintervals the query is divided into (must be greater equal 1)
+            outputFormat(str): output format of the query results - default format is json
+        Returns:
+            Query results in the requested outputFormat as string.
+            If the requested outputFormat is not supported None is returned.
         '''
         pageRecords=self.queryPages(askQuery, wiki, limit, showProgress, queryDivision)
-        print(outputFormat)
-        for pageRecord in pageRecords:
-            print(pageRecord)
-     
+        if outputFormat.lower() == "csv":
+            return self.convertToCSV(pageRecords)
+        elif outputFormat.lower() == "json":
+            return json.dumps(pageRecords)
+        else:
+            if self.debug:
+                print(f"Format {outputFormat} is not supported.")
+        return None
+
+    def convertToCSV(self, pageRecords, separator=";"):
+        """
+        Converts the given pageRecords into a str in csv format
+        ToDO: Currently does not support escaping of the separator and escaping of quotes
+        Args:
+            pageRecords: dict of dicts containing the printouts
+            separator(char):
+        Returns: str
+        """
+        res = ""
+        printedHeaders = False
+        for pageRecord in pageRecords.values():
+            if not printedHeaders:
+                for key in pageRecord.keys():
+                    res = f"{res}{key}{separator}"
+                res = f"{res[:-1]}\n"
+                printedHeaders = True
+            for printouts in pageRecord.values():
+                res = f"{res}{printouts}{separator}"
+            res = f"{res[:-1]}\n"   # remove last separator and end line
+        return res
+
+
     def queryPages(self,askQuery,wiki=None,limit=None,showProgress=False, queryDivision=1):
         '''
         query the given wiki for pagerecords matching the given askQuery
@@ -69,6 +108,7 @@ class WikiPush(object):
             wiki(wikibot): the wiki to query - use fromWiki if not specified
             limit(int): the limit for the query (optional)
             showProgress(bool): true if progress of the query retrieval should be indicated (default: one dot per 50 records ...)
+            queryDivision(int): Defines the number of subintervals the query is divided into (must be greater equal 1)
         Returns:
             list: a list of pageRecords matching the given askQuery
         '''
@@ -427,11 +467,42 @@ class WikiPush(object):
                 for item in warningsDict.items():
                     warnings.append(str(item))
             self.handleAPIWarnings(warnings,ignoreExists)
-       
 
-__version__ = "0.3.7"
+    def restore(self, pageTitles=None,backupPath=None):
+        """
+        restore given page titles from local backup
+        If no page titles are given the whole backup is restored.
+        Args:
+            pageTitles(list): a list of pageTitles to be restored to toWiki. If None -> full restore of backup
+            backupPath(str): path to backup location
+        """
+        if backupPath is None:
+            backupPath=self.getHomePath("wikibackup/%s" % self.toWikiId)
+        imageBackupPath = "%s/images" % backupPath
+        if pageTitles is None:
+            pageTitles = []
+            for path, subdirs, files in os.walk(backupPath):
+                for name in files:
+                    filename = os.path.join(path, name)[len(backupPath)+1:]
+                    if filename.endswith(".wiki"):
+                        pageTitles.append(filename[:-len(".wiki")])
+        total = len(pageTitles)
+        self.log("restoring %d pages from %s to %s" % (total, backupPath, self.toWikiId))
+        for i,pageTitle in enumerate(pageTitles):
+            try:
+                self.log("%d/%d (%4.0f%%): restore %s ..." % (i + 1, total, (i + 1) / total * 100, pageTitle),end='')
+                wikiFilePath = "%s/%s.wiki" % (backupPath, pageTitle)
+                with open(wikiFilePath, mode='r') as wikiFile:
+                    page_content = wikiFile.read()
+                    page = self.toWiki.getPage(pageTitle)
+                    page.edit(page_content, f"modified through wikirestore by {self.toWiki.wikiUser.user}")
+                self.log("✅")
+            except Exception as ex:
+                self.log("❌:%s" % str(ex) )
+
+__version__ = "0.3.9"
 __date__ = '2020-10-31'
-__updated__ = '2020-02-16'
+__updated__ = '2020-02-25'
 DEBUG=False
 
 def mainNuke(argv=None):
@@ -451,6 +522,9 @@ def mainUpload(argv=None):
 
 def mainBackup(argv=None):
     main(argv,mode='wikibackup')
+
+def mainRestore(argv=None):
+    main(argv,mode='wikirestore')
     
 def main(argv=None,mode='wikipush'): # IGNORE:C0111
     '''main program.'''
@@ -494,8 +568,9 @@ def main(argv=None,mode='wikipush'): # IGNORE:C0111
             parser.add_argument("-l", "--login", dest="login", action='store_true', help="login to source wiki for access permission")
             parser.add_argument("-s", "--source", dest="source", help="source wiki id", required=True)
             parser.add_argument("-wi", "--withImages", dest="withImages", action='store_true', help="copy images on the given pages")
+            parser.add_argument("--backupPath", dest="backupPath", help="path where the backup should be stored", required=False)
         elif mode=="wikinuke":
-            parser.add_argument("-f", "--force", dest="force", action='store_true', help="force to delete pages - default is 'dry' run only listing pages")            
+            parser.add_argument("-f", "--force", dest="force", action='store_true', help="force to delete pages - default is 'dry' run only listing pages")
         elif mode=="wikiedit":
             parser.add_argument("--search", dest="search", help="search pattern", required=True)
             parser.add_argument("--replace", dest="replace", help="replace pattern", required=True)
@@ -509,7 +584,11 @@ def main(argv=None,mode='wikipush'): # IGNORE:C0111
             parser.add_argument("--files", nargs='+', help="list of files to be uploaded", required=True)
             parser.add_argument("-f", "--force", dest="force", action='store_true', help="force to (re)upload existing files - default is false")            
             pass
-        if mode in  ["wikipush","wikiedit","wikinuke","wikibackup","wikiquery"]:
+        elif mode=="wikirestore":
+            parser.add_argument("--backupPath", dest="backupPath", help="path the backup is stored", required=False)
+            parser.add_argument("-s", "--source", dest="source", help="source wiki id", required=False)
+            parser.add_argument("-l", "--login", dest="login", action='store_true', help="login to source wiki for access permission")
+        if mode in  ["wikipush","wikiedit","wikinuke","wikibackup","wikiquery","wikirestore"]:
             parser.add_argument("--limit",dest="limit",type=int,help="limit for query")
             parser.add_argument("--progress",dest="showProgress",action='store_true',help="shows progress for query")
             parser.add_argument("-q", "--query", dest="query", help="select pages with given SMW ask query", required=False)
@@ -523,7 +602,7 @@ def main(argv=None,mode='wikipush'): # IGNORE:C0111
             parser.add_argument("-t", "--target", dest="target", help="target wiki id", required=True)    
         # Process arguments
         args = parser.parse_args(argv)
-        if args.queryDivision:
+        if hasattr(args,"queryDivision"):
             if args.queryDivision < 1:
                 raise ValueError("queryDivision argument must be greater equal 1")
         
@@ -538,6 +617,9 @@ def main(argv=None,mode='wikipush'): # IGNORE:C0111
             queryWiki=wikipush.fromWiki
         elif mode=="wikiupload":
             wikipush=WikiPush(None,args.target,debug=args.debug)
+        elif mode=="wikirestore":
+            wikipush=WikiPush(args.source,args.target,login=args.login,debug=args.debug)
+            queryWiki=wikipush.fromWiki
         else:
             wikipush=WikiPush(None,args.target,debug=args.debug)
             queryWiki=wikipush.toWiki
@@ -554,13 +636,20 @@ def main(argv=None,mode='wikipush'): # IGNORE:C0111
                     with open(args.queryFile, 'r') as queryFile:
                         query=queryFile.read()
                 if mode=="wikiquery":
-                    wikipush.formatQueryResult(query,wiki=queryWiki,limit=args.limit,showProgress=args.showProgress, queryDivision=args.queryDivision,outputFormat=args.format)    
+                    formatedQueryResults = wikipush.formatQueryResult(query,wiki=queryWiki,limit=args.limit,showProgress=args.showProgress, queryDivision=args.queryDivision,outputFormat=args.format)
+                    if formatedQueryResults:
+                        print(formatedQueryResults)
+                    else:
+                        print(f"Format {args.format} is not supported.")
                 else:     
                     pages=wikipush.query(query,wiki=queryWiki,queryField=args.queryField,limit=args.limit,showProgress=args.showProgress, queryDivision=args.queryDivision)
             if pages is None:
                 if mode=="wikiquery":
                     # we are finished
                     pass
+                if mode=="wikirestore":
+                    if args.pages is None and args.queryFile is None and args.query is None:
+                        wikipush.restore(pageTitles=None,backupPath=args.backupPath)
                 else:
                     raise Exception("no pages specified - you might want to use the -p, -q or --queryFile option")
             else:
@@ -572,12 +661,14 @@ def main(argv=None,mode='wikipush'): # IGNORE:C0111
                 if mode=="wikipush":
                     wikipush.push(pages,force=args.force,ignore=args.ignore,withImages=args.withImages)
                 elif mode=="wikibackup":
-                    wikipush.backup(pages,git=args.git,withImages=args.withImages)   
+                    wikipush.backup(pages,git=args.git,withImages=args.withImages,backupPath=args.backupPath)
                 elif mode=='wikinuke':
                     wikipush.nuke(pages,force=args.force)
                 elif mode=='wikiedit':
                     modify=WikiPush.getModify(args.search,args.replace)
                     wikipush.edit(pages,modify=modify,context=args.context,force=args.force)
+                elif mode=="wikirestore":
+                    wikipush.restore(pages,backupPath=args.backupPath)
                 else:
                     raise Exception("undefined wikipush mode %s" % mode)
         
